//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bcee.web;

import java.io.IOException;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebClientOptions;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableBody;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;

import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;

/**
 * Utility-Klasse fr die BCEE Website
 * 
 * @author Frank Mnster
 */
public final class BceeWebSiteUtils {

	/**
	 * Privater Konstruktor
	 */
	private BceeWebSiteUtils() {
		throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
	}

	/**
	 * Liefert einen instanziierten HtmlUnit WebClient
	 * 
	 * @return der WebClient
	 */
	/* default */static WebClient getWebClient() { // NOPMD

		final WebClient webcl = new WebClient();

		final WebClientOptions options = webcl.getOptions();

		options.setUseInsecureSSL(false);
		options.setJavaScriptEnabled(true);
		options.setThrowExceptionOnFailingStatusCode(false);
		options.setCssEnabled(true);
		options.setThrowExceptionOnScriptError(false);
		options.setRedirectEnabled(true);

		webcl.setAjaxController(new NicelyResynchronizingAjaxController());
		webcl.getCookieManager().setCookiesEnabled(true);

		Utils.setProxyCfg(webcl, Bcee.URL_LOGIN);

		return webcl;

	}

	/* default */static HtmlPage openAccount(final WebClient webClient, final BceeAccountInformation accountInfo) // NOPMD
			throws WebSiteException {

		// ffne Konto
		HtmlPage page;
		Boolean accountFound = false;

		try {
			page = webClient.getPage(Bcee.URL_UEBERSICHT);
			Logger.trace(page.asText());

			if (page.asText().contains("Finanzlage : Strukturierte Darstellung")) { //$NON-NLS-1$

				// Tabelle mit Konten
				final HtmlTable table = (HtmlTable) page.getElementById("tableSorterExtended"); //$NON-NLS-1$
				final HtmlTableBody tbBody = table.getBodies().get(0);

				for (final HtmlTableRow row : tbBody.getRows()) {
					Logger.debug(row.getCells().get(0).getFirstChild().toString());

					if (row.getCells().get(0).getFirstChild().toString().equals("Haben")) { //$NON-NLS-1$
						continue;
					}

					final String iban = row.getCells().get(1).asText().replaceAll("\\s", StringUtils.EMPTY); //$NON-NLS-1$
					Logger.debug("Vorhandene Konten: " + iban); //$NON-NLS-1$
					Logger.debug("Konto IBAN: " + accountInfo.getKonto().getIban()); //$NON-NLS-1$

					if (iban.equals(accountInfo.getKonto().getIban())) {
						accountFound = true;

						// Konto ffnen
						final DomElement anchor = row.getCells().get(0).getFirstElementChild();
						Logger.debug(anchor.toString());

						page = anchor.click();
						Logger.trace(page.asText());
						break;
					}
				}
			}
		} catch (FailingHttpStatusCodeException | IOException e) {
			final WebSiteException wse = new WebSiteException(e.getMessage(), WebSiteExceptionTypes.KONTO_NOT_FOUND);
			wse.addSuppressed(e);
			throw wse; // NOPMD
		}
		if (!accountFound) {
			// Wenn man hier ist, wurde das Konto nicht gefunden
			Logger.error(I18N.tr("BceeWebSiteUtils.accountNotFound")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSiteUtils.accountNotFound"), //$NON-NLS-1$
					WebSiteExceptionTypes.KONTO_NOT_FOUND);
		}

		return page;

	}

}
