//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.List;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.web.BshAccountInformation;
import de.frankmuenster.hibiscus.mahoe.bsh.web.BshBankMessages;
import de.frankmuenster.hibiscus.mahoe.bsh.web.BshWebSite;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.utils.TransactionSaver;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hier wird die eigentliche Arbeit gemacht.
 * 
 * @author Frank Mnster
 * 
 */
public class BshSynchronizeJobKontoauszug
		extends AbstractMahoeSynchronizeJobKontoauszug<BshAccountInformation, BshBankMessages> {

	/**
	 * Standartkonstruktor
	 */
	public BshSynchronizeJobKontoauszug() {
		super();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
	 */
	@Override
	protected void getWebSite() throws WebSiteException, ApplicationException {
		webSite = BshWebSite.getInstance(konto);
	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#login()
	 */
	@Override
	protected void login() throws WebSiteException, ApplicationException {
		final char[] password = PasswdCache.getPassword(SchwaebischHall.NAME, kundennummer);

		try {
			webSite.login(password);
		} catch (WebSiteException e) {
			if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
				// Passwort zurcksetzten
				PasswdCache.revoke(SchwaebischHall.NAME, kundennummer);
			}
			throw e;
		}
		Arrays.fill(password, ' ');
	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
	 */
	@Override
	protected void handleTransactions() throws WebSiteException, ApplicationException {

		monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.getBooked", //$NON-NLS-1$
				accountInfo.getLastTransactionDateAsString()));
		final List<Umsatz> booked = webSite.getBookedTransactions();

		// Debug-Ausgabe
		if (Logger.isLogging(Level.DEBUG)) {
			for (final Umsatz umsatz : booked) {

				String umsatzString;
				try {
					umsatzString = I18N.tr("SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
							umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
							Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
							umsatz.getKommentar());
					Logger.debug(umsatzString);
				} catch (RemoteException e) {
					Logger.error(I18N.tr("SynchronizeJobKontoauszug.createTransactionInfoStringFailed"), e); //$NON-NLS-1$
				}
			}
		}

		// Speicher gebuchte Umstze
		monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
		final int bookedCount = TransactionSaver.saveBooked(konto, booked);
		monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
		monitor.setPercentComplete(50);

		Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

	}

	/**
	 * @see AbstractMahoeSynchronizeJobKontoauszug#handleAccountSheets()
	 */
	@Override
	protected void handleAccountSheets() throws WebSiteException, ApplicationException {
		// TODO Auto-generated method stub

	}

}
