//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javax.annotation.Resource;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeJobProvider;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.utils.BlzToAdd;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobKontoauszug;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung des Job-Providers fr den Abruf von Kontoauszuegen der BSH.
 * 
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
@BlzToAdd
public class BshSynchronizeJobProviderKontoauszug implements MahoeSynchronizeJobProvider {

	@Resource
	private final MahoeSynchronizeBackend backend = null; // NOPMD

	/**
	 * Liste der von diesem Backend implementierten Jobs
	 */
	private final static List<Class<? extends SynchronizeJob>> JOBS = new ArrayList<Class<? extends SynchronizeJob>>();
	static {
		{
			BshSynchronizeJobProviderKontoauszug.JOBS.add(BshSynchronizeJobKontoauszug.class);
		}
	}

	/**
	 * Die vom Job-Provider explizit untersttzten Konto-Arten. Alle anderen
	 * machen andere Provider.
	 */
	private final static Set<KontoType> SUPPORTED = new HashSet<KontoType>();
	static {
		{
			BshSynchronizeJobProviderKontoauszug.SUPPORTED.add(KontoType.BAUSPAR);
		}
	}

	// Wir tragen uns in die untersttzen BLZ -List ein
	static {
		MahoeSynchronizeBackend.addBlzSupported(SchwaebischHall.BLZ, SchwaebischHall.NAME);
	}

	/**
	 * Standardkonstruktor
	 */
	public BshSynchronizeJobProviderKontoauszug() {
		// Absichtlich leer
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getSynchronizeJobs(de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	public List<SynchronizeJob> getSynchronizeJobs(final Konto konto) {

		final Class<SynchronizeJobKontoauszug> type = SynchronizeJobKontoauszug.class;

		final List<SynchronizeJob> jobs = new LinkedList<SynchronizeJob>();
		for (final Konto kto : this.backend.getSynchronizeKonten(konto)) {
			try {
				// Checken, ob das ein MahoeMain-Konto ist
				if (!this.backend.supports(type, konto)) {
					continue;
				}

				// Supporten wir das Konto
				if (!this.supports(type, konto)) {
					continue;
				}

				final SynchronizeOptions options = new SynchronizeOptions(kto); // NOPMD

				// Sync-Option zum Kontoauszugs-Abruf aktiv?
				if (!options.getSyncKontoauszuege()) {
					continue;
				}

				// erzeugt eine Instanz von BceeSynchronizeJobKontoauszug
				final SynchronizeJobKontoauszug job = this.backend.create(type, kto);
				job.setContext(SynchronizeJob.CTX_ENTITY, kto);
				jobs.add(job);
			} catch (RemoteException | ApplicationException e) {
				Logger.error(I18N.tr("BshSynchronizeJobProviderKontoauszug.getSynchnizedJobsFailed"), e); //$NON-NLS-1$
			}
		}

		return jobs;
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#getJobTypes()
	 */
	@Override
	public List<Class<? extends SynchronizeJob>> getJobTypes() {
		return BshSynchronizeJobProviderKontoauszug.JOBS;
	}

	/**
	 * @see de.willuhn.jameica.hbci.synchronize.SynchronizeJobProvider#supports(java.lang.Class,
	 *      de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	public boolean supports(final Class<? extends SynchronizeJob> type, final Konto konto) {

		boolean support = false;

		// Kein Konto angegeben. Dann gehen wir mal davon aus, dass es geht
		if (konto == null) {
			support = true;

		} else {
			KontoType kontoType = null;
			try {

				// Checken, ob es ein BSH-Konto ist, aber via BIC, nicht BLZ
				if (SchwaebischHall.BLZ.equals(konto.getBLZ())) {

					// Checken, ob das vielleicht ein nicht untersttztes Konto
					// ist
					kontoType = KontoType.find(konto.getAccountType());

					// Wenn kein konkreter Typ angegeben ist, dann untersttzen
					// wir es
					if (kontoType == null || BshSynchronizeJobProviderKontoauszug.SUPPORTED.contains(kontoType)) {
						support = true;
					}
				}
			} catch (RemoteException re) {
				Logger.error(I18N.tr("BshSynchronizeJobProviderKontoauszug.determineSupportForAccountTypeFailed") //$NON-NLS-1$
						, re);
			}
		}
		return support;

	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(final Object arg0) {
		// Umsaetze und Salden werden zum Schluss ausgefuehrt,
		// damit die oben gesendeten Ueberweisungen gleich mit
		// erscheinen, insofern die Bank das untersttzt.
		return 1;
	}

}
