//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.web;

import java.io.IOException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpHeaders;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.HttpMethod;
import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebRequest;
import com.gargoylesoftware.htmlunit.util.NameValuePair;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Account;
import de.frankmuenster.hibiscus.mahoe.bsh.json.BshResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Error;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Exemption;
import de.frankmuenster.hibiscus.mahoe.bsh.json.LoginResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.LogoffResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Status;
import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.AccountDetailData;
import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.Contract;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * BSH Bank Website
 * 
 * @author Frank Mnster
 * 
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass" })
public final class BshWebSite extends AbstractWebSite<BshAccountInformation, BshBankMessages> {

	/**
	 * 
	 */
	private static final String EUR_LF = " EUR\n"; //$NON-NLS-1$

	/**
	 * HTTP Header User-Agent Wert
	 */
	public static final String CUSTOM_USER_AGENT = "Jameica Plugin"; //$NON-NLS-1$

	/**
	 * HTTP Header User-Agent
	 */
	public static final String HEADER_USER_AGENT = "User-Agent"; //$NON-NLS-1$

	/**
	 * Post-Data Benutzerfeldname
	 */
	public static final String PARAM_KD_NUMER = "username"; //$NON-NLS-1$

	/**
	 * Post-Data Passwortfeldname
	 */
	public static final String PARAM_PASSWORD = "password"; //$NON-NLS-1$

	/**
	 * Post-Data Logoof-Feldname
	 */
	private static final String PARAM_LOGOFF = "logout"; //$NON-NLS-1$

	/**
	 * Jackson Mapper
	 */
	private static final ObjectMapper MAPPER = new ObjectMapper();

	/**
	 * JSON Timeout marker
	 */
	private static final CharSequence SESSION_TIMEOUT = "Session has a timeout"; //$NON-NLS-1$

	/**
	 * Sparphase
	 */
	private static final int SPAR_PHASE = 1;

	/**
	 * Dahrlehnphase
	 */
	private static final int DAHRLEHN_PHASE = 1;

	/**
	 * Unsere BshWebSite instance
	 */
	private static BshWebSite webSite;

	/**
	 * Die Kundennummer
	 */
	private final transient String kdNummer;

	/**
	 * Die Vertagdetails
	 */
	private transient AccountDetailData accountDetails;

	/**
	 * Liefert die Instance der WebSeite
	 * 
	 * @param konto
	 * @return die WebSeite
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public static BshWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {
		String kdNummer = null;
		String vtn = null;
		try {
			kdNummer = konto.getKundennummer();
			vtn = konto.getMeta(MahoeSynchronizeBackend.BSH_VTN, null);
		} catch (RemoteException e) {
			final String msg = I18N.tr("BceeWebSite.loginMissing"); // NOPMD //$NON-NLS-1$
			Logger.error(msg, e);
			throw new ApplicationException(msg, e);
		}

		if (BshWebSite.webSite == null) { // NOPMD
			BshWebSite.webSite = new BshWebSite(konto, kdNummer, vtn);
		} else {
			// Ist die Kundennummer dieselbe
			if (BshWebSite.webSite.kdNummer.equals(kdNummer)) {
				// Aktualisiere Startdatum der Umsatzabfrage/VTN und Konto
				final Date lastTransacDate = Utils.getLastUmsatzDate(konto, BshWebSite.webSite.getStartdate());
				BshWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
				BshWebSite.webSite.accountInfo.setVtn(vtn);
				BshWebSite.webSite.accountInfo.setKonto(konto);
			} else {
				// MAHOE-62 Check if logged in and do a log off first
				if (BshWebSite.webSite.isStillLoggedIn()) {
					Logger.debug(I18N.tr("BshWebSite.logoffDueToDifferentUser")); //$NON-NLS-1$
					BshWebSite.webSite.logoff();
				}
				BshWebSite.webSite = new BshWebSite(konto, kdNummer, vtn);
			}
		}
		return BshWebSite.webSite;
	}

	/**
	 * Liefert die Instance der WebSeite, nur mit Angabe der kdNummer.
	 * 
	 * @param kdNummer
	 * @return die WebSeite
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public static BshWebSite getInstance(final String kdNummer) throws WebSiteException, ApplicationException { // NOPMD

		if (BshWebSite.webSite == null || !BshWebSite.webSite.kdNummer.equals(kdNummer)) { // NOPMD
			BshWebSite.webSite = new BshWebSite(null, kdNummer, null);
		}
		return BshWebSite.webSite;

	}

	/**
	 * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der BSH.
	 * 
	 * @param konto
	 * @param kdNummer
	 * @param vtn
	 * @throws WebSiteException
	 * 
	 */
	private BshWebSite(final Konto konto, final String kdNummer, final String vtn) throws WebSiteException {
		super(konto);
		this.kdNummer = kdNummer;
		this.webClient = BshWebSiteUtils.getWebClient();

		this.accountInfo.setVtn(vtn);

	}

	/**
	 * @see AbstractWebSite#isStillLoggedIn()
	 */
	@Override
	public boolean isStillLoggedIn() throws WebSiteException {

		boolean isLoggedIn = false;

		Page responsePage;
		try {
			responsePage = this.webClient.getPage(SchwaebischHall.URL_CONTRACTS);
			if (BshWebSiteUtils.webResponseIsJson(responsePage.getWebResponse())) {
				Logger.debug("LoggedInTest: " + responsePage.getWebResponse().getContentAsString()); //$NON-NLS-1$
				if (!responsePage.getWebResponse().getContentAsString().contains(BshWebSite.SESSION_TIMEOUT)) {
					isLoggedIn = true;
				}
			}
		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("BshWebSite.connectionCheckFailed"), e); //$NON-NLS-1$
		}

		return isLoggedIn;
	}

	/**
	 * @see AbstractWebSite#login(char[], String...)
	 */
	@Override
	public void login(final char[] password, final String... loginParams) throws WebSiteException {

		Logger.debug(I18N.tr("BshWebSite.startLogin", this.kdNummer)); //$NON-NLS-1$

		if (loginParams.length != 0) {
			throw new WebSiteException(I18N.tr("BshWebSite.loginParameterCountWrong"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR);
		}

		try {

			final URL url = new URL(SchwaebischHall.URL_LOGIN_LOGOFF);
			final WebRequest requestSetting = new WebRequest(url, HttpMethod.POST);
			// requestSetting.setAdditionalHeader(HEADER_USER_AGENT,
			// CUSTOM_USER_AGENT);
			requestSetting.setAdditionalHeader(HttpHeaders.USER_AGENT, BshWebSite.CUSTOM_USER_AGENT);
			final List<NameValuePair> urlParameters = new ArrayList<>();
			urlParameters.add(new NameValuePair(BshWebSite.PARAM_KD_NUMER, this.kdNummer));
			urlParameters.add(new NameValuePair(BshWebSite.PARAM_PASSWORD, new String(password)));
			requestSetting.setRequestParameters(urlParameters);

			final Page redirectPage = this.webClient.getPage(requestSetting);

			if (BshWebSiteUtils.webResponseIsJson(redirectPage.getWebResponse())) {

				final BshResponse loginResponse = BshWebSite.MAPPER.readValue(redirectPage.getWebResponse()
						.getContentAsStream(), LoginResponse.class);

				if (loginResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = loginResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(loginResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.LOGIN_FAILED);
					}
				}

				// VTN schon vorhanden, nur wenn Konto vorhanden
				if (this.accountInfo.getKonto() != null) {
					this.checkVTN();
				}

			} else {
				final String msg = I18N.tr("BshWebSite.loadLoginRequestFailed"); // NOPMD //$NON-NLS-1$
				Logger.error(msg);
				throw new WebSiteException(msg, WebSiteExceptionTypes.LOGIN_ERROR);
			}

		} catch (FailingHttpStatusCodeException | IOException | ApplicationException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(e.getMessage(), WebSiteExceptionTypes.LOGIN_ERROR, e);
		}

	}

	/**
	 * @see AbstractWebSite#logoff()
	 */
	@Override
	public void logoff() throws WebSiteException {

		Logger.info("Schliee Bankverbindung."); //$NON-NLS-1$
		try {

			final URL url = new URL(SchwaebischHall.URL_LOGIN_LOGOFF);
			final WebRequest requestSetting = new WebRequest(url, HttpMethod.POST);
			// requestSetting.setAdditionalHeader(HEADER_USER_AGENT,
			// CUSTOM_USER_AGENT);
			requestSetting.setAdditionalHeader(HttpHeaders.USER_AGENT, BshWebSite.CUSTOM_USER_AGENT);
			final List<NameValuePair> urlParameters = new ArrayList<>();
			// urlParameters.add(new NameValuePair(PARAM_KD_NUMER, kdNummer));
			urlParameters.add(new NameValuePair(BshWebSite.PARAM_LOGOFF, new String("1"))); //$NON-NLS-1$
			requestSetting.setRequestParameters(urlParameters);

			final Page redirectPage = this.webClient.getPage(requestSetting);

			if (BshWebSiteUtils.webResponseIsJson(redirectPage.getWebResponse())) {

				final BshResponse logoffResponse = BshWebSite.MAPPER.readValue(redirectPage.getWebResponse()
						.getContentAsStream(), LogoffResponse.class);

				if (logoffResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = logoffResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
					}
				} else {
					Logger.info(I18N.tr("BshWebSite.logoffSuccessful")); //$NON-NLS-1$
				}

			} else {
				final String msg = I18N.tr("BshWebSite.loadLogoffRequestFailed"); // NOPMD //$NON-NLS-1$
				Logger.error(msg);
			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
		}

	}

	/**
	 * @see AbstractWebSite#getAccountInformation()
	 */
	@Override
	public BshAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

		this.checkVTN();

		// allg. Vertragsdaten
		final Account account = BshWebSiteUtils.getAccount(this.webClient, this.accountInfo.getVtn());
		this.accountInfo.setBsn(account.getBsn());
		this.accountInfo.setType(account.getType());
		this.accountInfo.setContractLetter(account.getContractLetter());
		this.accountInfo.setContractNumber(account.getContractNumber());

		// Vertragsdetails
		this.accountDetails = BshWebSiteUtils.getAccountDetails(this.webClient, this.accountInfo.getVtn());

		// Freistellungsauftrag
		final Exemption exemption = BshWebSiteUtils.getExemption(this.webClient,
				account.getCoreId() + account.getContractLetter());
		String exemptionStr = StringUtils.EMPTY;
		if (exemption != null) {
			final StringBuilder exemptionBuilder = new StringBuilder();
			exemptionBuilder
					.append(I18N.tr("BshWebSite.exemption")).append(exemption.getTariffamount()) //$NON-NLS-1$
					.append(I18N.tr("BshWebSite.suggestedExemption")).append(exemption.getSuggestedamount()) //$NON-NLS-1$
					.append(" EUR)\n") //$NON-NLS-1$
					.append(I18N.tr("BshWebSite.abticipatedInteresetAmount")).append(exemption.getAnticipatedamount()).append(BshWebSite.EUR_LF); //$NON-NLS-1$
			exemptionStr = exemptionBuilder.toString();
		}

		this.accountInfo.setBsn(this.accountDetails.getCoreData().getBsn());
		this.accountInfo.setSaldo(this.accountDetails.getContract().getCurrentbalance().doubleValue());

		final Contract contractInfo = this.accountDetails.getContract();
		String comment = StringUtils.EMPTY;
		if (this.accountDetails.getContractphase() == BshWebSite.SPAR_PHASE) {
			comment = I18N.tr("BshWebSite.commetTarif") + contractInfo.getTarif() + "\n" //$NON-NLS-1$ //$NON-NLS-2$
					+ I18N.tr("BshWebSite.commentSum") + contractInfo.getSaveSum() //$NON-NLS-1$
					+ BshWebSite.EUR_LF
					+ I18N.tr("BshWebSite.commentSavingInterest") + contractInfo.getSavingInterestRate() + " %\n" //$NON-NLS-1$ //$NON-NLS-2$ 
					+ I18N.tr("BshWebSite.commentRatingNumber") //$NON-NLS-1$
					+ contractInfo.getRatingNumber()
					+ "\n" + I18N.tr("BshWebSite.commentSubityCharge") //$NON-NLS-1$ //$NON-NLS-2$
					+ this.accountDetails.getCoreData().getSubsidycharge()
					+ BshWebSite.EUR_LF
					+ I18N.tr("BshWebSite.commentSubitySum") //$NON-NLS-1$
					+ this.accountDetails.getSubsidy().getSum() + BshWebSite.EUR_LF + exemptionStr
					+ I18N.tr("BshWebSite.commentAssignmentGoal") //$NON-NLS-1$ 
					+ this.accountDetails.getAssignmentgoal().getText() + "\n"; //$NON-NLS-1$
		} else if (this.accountDetails.getContractphase() == BshWebSite.DAHRLEHN_PHASE) {
			comment = I18N.tr("BshWebSite.commetTarif") + contractInfo.getTarif() + "\n" //$NON-NLS-1$ //$NON-NLS-2$
					+ I18N.tr("BshWebSite.commentSum") + contractInfo.getSaveSum() //$NON-NLS-1$
					+ BshWebSite.EUR_LF
					+ I18N.tr("BshWebSite.commentSavingInterest") + contractInfo.getSavingInterestRate() + " %\n" //$NON-NLS-1$//$NON-NLS-2$
					+ I18N.tr("BshWebSite.commentGrantEnd") + contractInfo.getGrantend(); //$NON-NLS-1$
		}
		this.accountInfo.setComment(comment);

		return this.accountInfo;
	}

	/**
	 * @see AbstractWebSite#getPreBookedTransactions()
	 */
	@Override
	public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {
		// BSH hat keine vorgemerkten Umstze
		return null;
	}

	/**
	 * @see AbstractWebSite#getBookedTransactions()
	 */
	@Override
	public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

		if (this.accountDetails == null) {
			this.checkVTN();
			this.accountDetails = BshWebSiteUtils.getAccountDetails(this.webClient, this.accountInfo.getVtn());
		}

		return BshWebSiteUtils.getBookedTransctions(this.accountDetails);

	}

	/**
	 * @see AbstractWebSite#savePDF(de.frankmuenster.hibiscus.mahoe.web.InterfaceWebAccountInformation)
	 */
	@Override
	public int savePDF(final BshAccountInformation accountInfo) throws PdfSaverException {
		Logger.info(I18N.tr("BshWebSite.saveCheetsNotSupportet")); //$NON-NLS-1$
		return 0;
	}

	/**
	 * @see AbstractWebSite#createSepaTransfer(AuslandsUeberweisung)
	 */
	@Override
	public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung) throws WebSiteException,
			ApplicationException {
		Logger.info(I18N.tr("BshWebSite.transferNotSupported")); //$NON-NLS-1$
	}

	/**
	 * @see AbstractWebSite#getWebClient()
	 */
	@Override
	protected WebClient getWebClient() {
		return this.webClient;
	}

	/**
	 * @see AbstractWebSite
	 */
	@Override
	protected Calendar getStartdate() {
		// Bei BSH knnen wir alles nehmen
		return null;
	}

	/**
	 * Ist die VTN schon in den Sync-Parametern eingetragen? Wenn, dann machen
	 * wir das jetzt.
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	private void checkVTN() throws WebSiteException, ApplicationException {

		if (StringUtils.isEmpty(BshWebSite.webSite.accountInfo.getVtn())) {

			final List<Account> accounts = BshWebSiteUtils.getAccounts(this.webClient);
			for (final Account account : accounts) {
				Logger.debug(account.toString());

				try {

					final String testString = this.accountInfo.getKonto().getKontonummer().trim()
							+ this.accountInfo.getKonto().getUnterkonto().trim();
					final String contract = account.getBsn().replaceAll(" ", StringUtils.EMPTY) + "%" //$NON-NLS-1$ //$NON-NLS-2$
							+ account.getType();

					if (contract.startsWith(testString)) {
						// Richtigen Vertrag gefunden --> vtn eintragen
						this.accountInfo.setVtn(account.getVtn());
						break;
					}
				} catch (RemoteException e) {
					Logger.error(e.getMessage());
					throw new ApplicationException(I18N.tr("BshWebSite.readAccountInfoFailed"), e); //$NON-NLS-1$
				}
			}

			if (StringUtils.isEmpty(this.accountInfo.getVtn())) {
				Logger.error(I18N.tr("BshWebSite.getVTNFailed")); //$NON-NLS-1$
				throw new ApplicationException(I18N.tr("BshWebSite.getVTNFailed")); //$NON-NLS-1$
			}

		}

	}

	/**
	 * Liefert alle Vertrge unter der Kundennummer
	 * 
	 * @return eine List emit allen Vertrgen
	 * @throws WebSiteException
	 * @throws OperationCanceledException
	 */
	public List<Account> getAccounts() throws OperationCanceledException, WebSiteException {

		if (!this.isStillLoggedIn()) {

			final char[] password = PasswdCache.getPassword(SchwaebischHall.NAME, this.kdNummer);

			try {
				BshWebSite.webSite.login(password);
			} catch (WebSiteException e) {
				if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
					// Passwort zurcksetzten
					PasswdCache.revoke(SchwaebischHall.NAME, this.kdNummer);
				}
				throw e;
			}
			Arrays.fill(password, ' ');
		}

		return BshWebSiteUtils.getAccounts(this.webClient);
	}

	/**
	 * Liefert die IBAN zu einem Vertrag
	 * 
	 * @param bsnKurz
	 * 
	 * @param blz
	 *            die Bankleitzahl
	 * 
	 * @return die IBAN
	 * 
	 * @throws WebSiteException
	 * @throws OperationCanceledException
	 */
	public String getIban(final String bsnKurz, final String blz) throws OperationCanceledException, WebSiteException {

		if (!this.isStillLoggedIn()) {

			final char[] password = PasswdCache.getPassword(SchwaebischHall.NAME, this.kdNummer);

			try {
				BshWebSite.webSite.login(password);
			} catch (WebSiteException e) {
				if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
					// Passwort zurcksetzten
					PasswdCache.revoke(SchwaebischHall.NAME, this.kdNummer);
				}
				throw e;
			}
			Arrays.fill(password, ' ');
		}

		return BshWebSiteUtils.getIban(this.webClient, bsnKurz, blz);
	}

}
