//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.santander.jobs;

import java.rmi.RemoteException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug;
import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.santander.web.AbstractSantanderWebSite;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderAccountInformation;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankEnum;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankMessages;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.PasswdCache;
import de.frankmuenster.hibiscus.mahoe.utils.TransactionSaver;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaver;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Hier wird die eigentliche Arbeit gemacht.
 * 
 * @author Frank Mnster
 * 
 */
public class SantanderSynchronizeJobKontoauszug extends
		AbstractMahoeSynchronizeJobKontoauszug<SantanderAccountInformation, SantanderBankMessages> {

	/**
	 * Standartkonstruktor
	 */
	public SantanderSynchronizeJobKontoauszug() {
		super();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#getWebSite()
	 */
	@Override
	protected void getWebSite() throws WebSiteException, ApplicationException {
		this.webSite = AbstractSantanderWebSite.getInstance(this.konto);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#login()
	 */
	@Override
	protected void login() throws WebSiteException, ApplicationException {

		String bankName;
		try {
			bankName = this.konto.getMeta(MahoeSynchronizeBackend.BANK_NAME, SantanderBankEnum.CONSUMER.getIdString());
			bankName = "Santander " + SantanderBankEnum.find(bankName).getBankName(); //$NON-NLS-1$
		} catch (RemoteException e) {
			throw new ApplicationException(e);
		}
		final char[] password = PasswdCache.getPassword(bankName, this.kundennummer);
		try {
			this.webSite.login(password);
		} catch (WebSiteException e) {
			if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)) {
				// Passwort zurcksetzen
				PasswdCache.revoke(bankName, this.kundennummer);
			}
			throw e;
		}
		Arrays.fill(password, ' ');
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#handleTransactions()
	 */
	@Override
	protected void handleTransactions() throws WebSiteException, ApplicationException {

		try {
			if (KontoType.find(this.konto.getAccountType()).equals(KontoType.KREDITKARTE)) {

				this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.getPreBooked")); //$NON-NLS-1$
				final List<Umsatz> preBooked = this.webSite.getPreBookedTransactions();

				// Debug-Ausgabe
				if (Logger.isLogging(Level.DEBUG)) {
					for (final Umsatz umsatz : preBooked) {

						String umsatzString;
						try {
							umsatzString = I18N.tr(
									"SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
									umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
									Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
									umsatz.getKommentar());
							Logger.debug(umsatzString);
						} catch (RemoteException e) {
							Logger.error(I18N.tr("SynchronizeJobKontoauszug.createTransactionInfoStringFailed"), e); //$NON-NLS-1$
						}
					}
				}

				// Speicher der vorgemerkten Umstze
				this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.savingPreBooked")); //$NON-NLS-1$
				Collections.reverse(preBooked);
				final int preBookedCount = TransactionSaver.savePreBooked(this.konto, preBooked, false);
				this.monitor.setStatusText(I18N.tr(
						"SantanderSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$
				this.monitor.setPercentComplete(50);

				Logger.info(I18N.tr(
						"SantanderSynchronizeJobKontoauszug.savedPreBooked", Integer.toString(preBookedCount))); //$NON-NLS-1$

			}
		} catch (RemoteException e) {
			Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.getPreBookedFailed"), e); //$NON-NLS-1$
		}

		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.getBooked", //$NON-NLS-1$
				this.accountInfo.getLastTransactionDateAsString()));
		final List<Umsatz> booked = this.webSite.getBookedTransactions();

		// Debug-Ausgabe
		if (Logger.isLogging(Level.DEBUG)) {
			for (final Umsatz umsatz : booked) {

				String umsatzString;
				try {
					umsatzString = I18N.tr(
							"SynchronizeJobKontoauszug.transactionInfoLog", //$NON-NLS-1$
							umsatz.getDatum().toString(), umsatz.getZweck(), umsatz.getZweck2(),
							Double.toString(umsatz.getBetrag()), Double.toString(umsatz.getSaldo()),
							umsatz.getKommentar());
					Logger.debug(umsatzString);
				} catch (RemoteException e) {
					Logger.error(I18N.tr("SynchronizeJobKontoauszug.createTransactionInfoStringFailed"), e); //$NON-NLS-1$
				}
			}
		}

		// Speicher gebuchte Umstze
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savingBooked")); //$NON-NLS-1$
		Collections.reverse(booked);
		final int bookedCount = TransactionSaver.saveBooked(this.konto, booked, false);
		this.monitor.setStatusText(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$
		this.monitor.setPercentComplete(50);

		Logger.info(I18N.tr("SynchronizeJobKontoauszug.savedBooked", Integer.toString(bookedCount))); //$NON-NLS-1$

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.AbstractMahoeSynchronizeJobKontoauszug#handleAccountSheets()
	 */
	@Override
	protected void handleAccountSheets() throws WebSiteException, ApplicationException {

		try {

			final boolean savePdf = "JA".equals( //$NON-NLS-1$ // NOPMD
					this.konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF, "NEIN").toUpperCase()); //$NON-NLS-1$
			final String savePdfPath = this.konto.getMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, StringUtils.EMPTY);

			if (savePdf) {
				Logger.info(I18N.tr("SantanderSynchronizeJobKontoauszug.savePDF")); //$NON-NLS-1$
				String folder = savePdfPath;
				if (StringUtils.isEmpty(folder)) {
					// Directory Fenster ffnen
					folder = PdfSaver.getPath();
					if (!StringUtils.isEmpty(folder)) {
						try {
							this.konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, folder);
						} catch (RemoteException e) {
							Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.newSavePathStoreFailed"), //$NON-NLS-1$
									e);
						}
					}

				}
				if (StringUtils.isEmpty(folder)) {
					Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.noSavePathGiven")); //$NON-NLS-1$
				} else {
					this.accountInfo.setSavePath(folder);
					final int savedCount = this.webSite.savePDF(this.accountInfo);
					if (savedCount == 0) {
						this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.noNewAccountSheets")); //$NON-NLS-1$
					} else {
						this.monitor
								.setStatusText(I18N
										.tr("SantanderSynchronizeJobKontoauszug.savingNewSheetsIn", Integer.toString(savedCount), savePdfPath)); //$NON-NLS-1$

						final String text = I18N.tr("SantanderSynchronizeJobKontoauszug.newAccountSheetsAvailableMsg"); //$NON-NLS-1$
						Application.getMessagingFactory().sendMessage(
								new StatusBarMessage(text, StatusBarMessage.TYPE_SUCCESS)); // NOPMD
					}
					this.monitor.setPercentComplete(90);
				}
			} else {
				this.monitor.setStatusText(I18N.tr("SantanderSynchronizeJobKontoauszug.savePdfDisabled")); //$NON-NLS-1$
				Logger.info(I18N.tr("SantanderSynchronizeJobKontoauszug.savePdfDisabled")); //$NON-NLS-1$
				this.monitor.setPercentComplete(90);
			}
		} catch (RemoteException | PdfSaverException e) {
			Logger.error(I18N.tr("SantanderSynchronizeJobKontoauszug.savePdfFailed"), e); //$NON-NLS-1$
			throw new ApplicationException(e.getMessage(), e);
		}

	}

}
