//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.santander.web;

import org.apache.commons.lang.StringUtils;

/**
 * Enum fr die Santander Banken Consumer und Direkt
 * 
 * @author Frank Mnster
 * 
 */
public enum SantanderBankEnum {

	/**
	 * Consumer Bank
	 */
	CONSUMER("consumer", "Consumer Bank", //$NON-NLS-1$ //$NON-NLS-2$
			"https://securebank.santander.de/IRALOG_SCB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_logonPinUser", //$NON-NLS-1$
			"https://securebank.santander.de/EBANDE_PGSCB_ENS/BtoChannelDriver.ssobto?dse_operationName=globalPositionOwnerOP", //$NON-NLS-1$
			"https://securebank.santander.de/EBANDE_LGSSCB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_inBox"), //$NON-NLS-1$
	/**
	 * Direkt Bank
	 */
	DIREKT("direkt", "Direkt Bank", //$NON-NLS-1$ //$NON-NLS-2$
			"https://onlinebanking.santander-direkt.de/IRALOG_SDB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_logonPinUser", //$NON-NLS-1$
			"https://onlinebanking.santander-direkt.de/EBANDE_PGSDB_ENS/BtoChannelDriver.ssobto?dse_operationName=globalPositionOwnerOP", //$NON-NLS-1$
			"https://onlinebanking.santander-direkt.de/EBANDE_LGSSDB_ENS/BtoChannelDriver.ssobto?dse_operationName=OP_inBox"); //$NON-NLS-1$

	/**
	 * Der Identifikations-String
	 */
	private String idString;

	/**
	 * Der Name der Bank
	 */
	private String bankName;

	/**
	 * Login URL
	 */
	private String loginUrl;

	/**
	 * &Uuml;bersichtseite URL
	 */
	private String overviewUrl;

	/**
	 * PostBox URL
	 */
	private String postBoxUrl;

	/**
	 * Aufz&auml;hlung der Santanderbanken
	 * 
	 * @param idString
	 *            der Identifikations-String.
	 * @param bankName
	 *            der Name der Bank.
	 * @param loginUrl
	 *            die Login URL
	 * @param overviewUrl
	 *            die Finanz&uuml;bersicht URL
	 * @param postBoxUrl
	 *            die PostBox URL
	 */
	private SantanderBankEnum(final String idString, final String bankName, final String loginUrl,
			final String overviewUrl, final String postBoxUrl) {
		this.idString = idString;
		this.bankName = bankName;
		this.loginUrl = loginUrl;
		this.overviewUrl = overviewUrl;
		this.postBoxUrl = postBoxUrl;
	}

	/**
	 * Liefert den Identifikations-String.
	 * 
	 * @return idString der Identifikations-String.
	 */
	public String getIdString() {
		return idString;
	}

	/**
	 * Liefert den Bank Namen.
	 * 
	 * @return den Bank Namen.
	 */
	public String getBankName() {
		return this.bankName;
	}

	/**
	 * Liefert die Login URL.
	 * 
	 * @return die Login URL.
	 */
	public String getLoginUrl() {
		return this.loginUrl;
	}

	/**
	 * Liefert die Login URL.
	 * 
	 * @return die Login URL.
	 */
	public String getOverviewUrl() {
		return this.overviewUrl;
	}

	/**
	 * Liefert die PostBox URL.
	 * 
	 * @return die PostBox URL.
	 */
	public String getPostBoxUrl() {
		return this.postBoxUrl;
	}

	/**
	 * Ermittelt BankTyp.
	 * 
	 * @param idString
	 *            der Bank. Kann NULL sein.
	 * @return der ID-String oder NULL, wenn der Code nicht bekannt ist.
	 */
	public static SantanderBankEnum find(final String idString) {

		SantanderBankEnum santanderBank = null;
		if (StringUtils.trimToNull(idString) != null) {
			for (final SantanderBankEnum bankName : values()) {
				if (bankName.getIdString().equals(idString)) {
					santanderBank = bankName;
					break;
				}
			}
		}
		return santanderBank;
	}

	/**
	 * @see java.lang.Enum#toString()
	 */
	@Override
	public String toString() {
		return this.getBankName();
	}
}