//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.targobank.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountException;
import de.frankmuenster.hibiscus.mahoe.assistant.InterfaceCreateAccountAssistant;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.hbci.HBCI;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * TargoBnak Konto-Anlagen-Assistent
 * 
 * @author Frank Mnster
 * 
 */
public final class TargoBankCreateAccountAssistent implements InterfaceCreateAccountAssistant {

	/**
	 * Standardkonstruktor
	 */
	public TargoBankCreateAccountAssistent() {
		// Absichtlich leer
	}

	/**
	 * @see InterfaceCreateAccountAssistant#createAccount()
	 */
	@Override
	public Konto createAccount() throws CreateAccountException, OperationCanceledException {

		final TargoBankCreateAccountDialog infoDialog = new TargoBankCreateAccountDialog();
		TargoBankCreateInfo info = null;
		try {
			info = infoDialog.open();
		} catch (OperationCanceledException e) { // NOPMD
			throw e;
		} catch (Exception e) { // NOPMD
			Logger.warn(I18N.tr("TargoBankCreateAccountAssistent.openDialogFailedLog", //$NON-NLS-1$
					e.getMessage()));
			throw new CreateAccountException(I18N.tr("TargoBankCreateAccountAssistent.openDialogFailed"), e); //$NON-NLS-1$
		}
		if (StringUtils.isEmpty(info.getCardNumber()) || StringUtils.isEmpty(info.getCompany())) {
			throw new CreateAccountException(I18N.tr("TargoBankCreateAccountAssistent.missingParameters")); //$NON-NLS-1$
		}
		return createKonto(info);

	}

	private Konto createKonto(final TargoBankCreateInfo info) throws CreateAccountException {

		HBCIDBService dbService = null;
		try {
			dbService = (HBCIDBService) Application.getServiceFactory().lookup(HBCI.class, "database"); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(I18N.tr("TargoBankCreateAccountAssistent.noDbAccess"), e); //$NON-NLS-1$
			throw new CreateAccountException(I18N.tr("TargoBankCreateAccountAssistent.noDbAccess"), //$NON-NLS-1$
					e);
		}
		Konto konto;
		try {
			konto = dbService.createObject(Konto.class, null);

			konto.setName(info.getOwner());
			konto.setBezeichnung(info.getCompany());

			konto.setKontonummer(TargoBank.KK_KTO);
			konto.setIban(TargoBank.KK_IBAN);
			konto.setKundennummer(info.getLogin());
			konto.setUnterkonto(StringUtils.EMPTY);
			konto.setBLZ(TargoBank.BLZ);
			konto.setAccountType(KontoType.KREDITKARTE.getValue());
			konto.setFlags(Konto.FLAG_NONE);
			konto.setKommentar(info.getComment());

			konto.setBackendClass(MahoeSynchronizeBackend.BACKEND_NAME);
			konto.store();

			// Setze META Daten
			konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, info.getCompany());
			konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, info.getCardNumber());
			konto.setMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, info.getCardAccount());
			// Kontoauszugebehandlung
			final String savePdf = info.getSavePDF() ? "JA" : "NEIN"; //$NON-NLS-1$ //$NON-NLS-2$
			konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF, savePdf);
			konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, info.getSaveFolder());

			// Sync-Options setzen
			final SynchronizeOptions sync = new SynchronizeOptions(konto);
			sync.setAutoSaldo(false);
			sync.setSyncAuslandsUeberweisungen(false);
			sync.setSyncKontoauszuege(true);
			sync.setSyncOffline(false);
			sync.setSyncSaldo(true);
			sync.setSyncSepaDauerauftraege(false);
			sync.setSyncSepaLastschriften(false);
		} catch (RemoteException | ApplicationException e) {
			Logger.error(I18N.tr("TargoBankCreateAccountAssistent.createAccountFailedLog", //$NON-NLS-1$
					e.getMessage()), e);
			throw new CreateAccountException(I18N.tr("TargoBankCreateAccountAssistent.createAccountFailed"), //$NON-NLS-1$
					e);
		}

		return konto;
	}

}
