//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.targobank.web;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlDivision;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableBody;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;

import de.frankmuenster.hibiscus.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.hibiscus.mahoe.messages.MahoeMessage;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * List und Speichert Bank-Nachrichten
 * 
 * @author Frank Mnster
 * 
 */
public class TargoBankMessages extends AbstractBankMessages {

	/**
	 * 
	 */
	private static final String NO_MESSAGES = "Sie haben keine Mitteilung."; //$NON-NLS-1$

	/**
	 * Erstellt ein neues Objekt
	 * 
	 * @param webclient
	 */
	public TargoBankMessages(final WebClient webclient) {
		super(webclient);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.messages.AbstractBankMessages#readMessages(java.util.Date)
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	@Override
	public void readMessages(final Date startDate) throws WebSiteException, ApplicationException {

		Date sDate;

		// Startdatum ermitteln
		if (startDate == null) {
			final Calendar cal = Calendar.getInstance();
			cal.clear(Calendar.HOUR);
			cal.clear(Calendar.MINUTE);
			cal.clear(Calendar.SECOND);
			cal.set(Calendar.YEAR, cal.get(Calendar.YEAR) - 1);
			sDate = cal.getTime();
		} else {
			sDate = startDate;
		}
		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm", //$NON-NLS-1$
				Locale.GERMAN);

		try {
			final HtmlPage msgPage = webclient.getPage(TargoBank.URL_MESSAGES);
			final HtmlDivision tableDiv = (HtmlDivision) msgPage.getElementById("MailsTable"); //$NON-NLS-1$
			final HtmlTable table = (HtmlTable) tableDiv.getElementsByTagName("table").get(0); //$NON-NLS-1$
			final HtmlTableBody msgTable = table.getBodies().get(0);

			HtmlAnchor subjectAnchor = null;
			for (final HtmlTableRow row : msgTable.getRows()) {

				// MantisBT-076
				if (row.asText().contains(NO_MESSAGES)) {
					Logger.info(I18N.tr("TargoBankMessages.noNewMessages")); //$NON-NLS-1$

				} else {
					final String dateString = row.getCell(3).asText();
					final Date date = sdf.parse(dateString);
					if (date.after(sDate)) {
						subjectAnchor = (HtmlAnchor) row.getCell(2).getElementsByTagName("a").get(0); //$NON-NLS-1$
						final String subject = row.getCell(2).asText();
						if (!MahoeMessage.existsMessage(TargoBank.BLZ, date, subject)) {
							// Nachricht in Liste einfgen
							addMessage(subjectAnchor, date, subject);
						}
					}
				}

			}

		} catch (FailingHttpStatusCodeException |

				IOException e) {
			Logger.error(I18N.tr("TargoBankMessages.openMessagePageFailed"), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("TargoBankMessages.openMessagePageFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.PAGE_NOT_FOUND, e);
		} catch (ParseException e) {
			Logger.error(I18N.tr("TargoBankMessages.parseDateFailed"), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("TargoBankMessages.parseDateFailed"), e); //$NON-NLS-1$
		}

	}

	/**
	 * Fgt die Bank-Nachricht der MahoeNachrichten Liste hinzu.
	 * 
	 * @param subjectAnchor
	 *            der Link zur Nachrichten Datei Seite
	 * @param date
	 *            das Datum
	 * @param subject
	 *            der Betreff
	 * @throws WebSiteException
	 *             falls der Link nicht geffnet werden kann
	 */
	private void addMessage(final HtmlAnchor subjectAnchor, final Date date, final String subject)
			throws WebSiteException {

		Logger.debug("Lese Nachricht"); //$NON-NLS-1$

		String msgString = StringUtils.EMPTY;

		try {

			// Reload webpage
			HtmlPage msgPage = webclient.getPage(TargoBank.URL_MESSAGES);
			final DomElement anchor = msgPage.getElementById(subjectAnchor.getId());

			msgPage = anchor.click();

			Logger.debug(msgPage.asText());

			final HtmlDivision tableDiv = (HtmlDivision) msgPage.getElementById("F1:expContent"); //$NON-NLS-1$
			final HtmlTable table = (HtmlTable) tableDiv.getElementsByTagName("table").get(0); //$NON-NLS-1$
			final HtmlTableBody msgTable = table.getBodies().get(0);
			for (final HtmlTableRow row : msgTable.getRows()) {
				final String rowContent = row.getCell(0).asText();
				if (rowContent.startsWith("Nachricht")) { //$NON-NLS-1$
					msgString = row.getCell(1).asText();
					break;
				}
			}

			final MahoeMessage message = new MahoeMessage(TargoBank.BLZ, date, subject, msgString);
			messages.add(message);

		} catch (IOException e) {
			Logger.error(I18N.tr("TargoBankMessages.openMessageDetailsFailed"), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("TargoBankMessages.openMessageDetailsFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.PAGE_NOT_FOUND, e);
		}

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.messages.InterfaceBankMessages#markAsRead(de.frankmuenster.hibiscus.mahoe.messages.MahoeMessage)
	 */
	@Override
	public void markAsRead(final MahoeMessage message) throws ApplicationException {
		// Wir bei der TargoBank nicht bentigt, da wir auf den Link 'Klicken'.

	}
}
