//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.utils;

import java.io.File;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.ProxySelector;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.DefaultCredentialsProvider;
import com.gargoylesoftware.htmlunit.ProxyConfig;
import com.gargoylesoftware.htmlunit.WebClient;

import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.jameica.security.JameicaAuthenticator;
import de.willuhn.jameica.security.Login;
import de.willuhn.jameica.security.Wallet;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Utility-Klasse
 * 
 * @author Frank Mnster
 * 
 */
@SuppressWarnings("PMD.ExcessiveImports")
final public class Utils {

	/**
	 * Der JameicaAuthenticator
	 */
	private final static JameicaAuthenticator J_AUTHENTICATOR = new JameicaAuthenticator();

	/**
	 * Hibiscus Umsatz max. Zeilenlnge
	 */
	private final static int MAX_ROW_LENGTH = 35;

	/**
	 * Privater Konstruktor fr die Utils-Klasse
	 */
	private Utils() {
		throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
	}

	/**
	 * Maskiert eine Karten-Nummer und zeigt nur die letzten 4 Ziffern
	 * 
	 * @param kartenNummer
	 * @return die maskierte Kartennummer
	 */
	public static String maskCardNummer(final String kartenNummer) {

		final StringBuilder maskNumber = new StringBuilder(kartenNummer.length());
		maskNumber.append("XXXXXXXXXXXXXXXX".substring(0, kartenNummer.length() - 4)) //$NON-NLS-1$
				.append(kartenNummer.substring(kartenNummer.length() - 4));

		return maskNumber.toString();

	}

	/**
	 * Setzt die Proxy-Einstellungen
	 * 
	 * @param webClient
	 *            der WebClient
	 * @param url
	 *            die URL fr den ein Proxy bentigt wird
	 */
	public static void setProxyCfg(final WebClient webClient, final String url) {
		final boolean useSystem = Application.getConfig().getUseSystemProxy();

		if (useSystem) {
			Utils.useSystemProxy(webClient, url);
		} else {
			Utils.useManualProxy(webClient);
		}
	}

	/**
	 * Versucht die Proxy-Einstellungen vom System zu bernehmen.
	 * 
	 * @param webClient
	 *            der WebClient
	 * @param url
	 *            die URL fr den ein Proxy bentigt wird
	 */
	private static void useSystemProxy(final WebClient webClient, final String url) {

		final ProxyConfig proxyConfig = new ProxyConfig();

		try {
			final List<java.net.Proxy> proxies = ProxySelector.getDefault().select(new URI(url));
			Logger.info(I18N.tr("Utils.usingSystemProxy", proxies.toString())); //$NON-NLS-1$
			for (final Proxy proxy : proxies) {
				if (proxy.type() == Proxy.Type.HTTP && proxy.address() instanceof InetSocketAddress) {
					final InetSocketAddress addr = (InetSocketAddress) proxy.address();
					proxyConfig.setProxyHost(addr.getHostString());
					proxyConfig.setProxyPort(addr.getPort());
					webClient.getOptions().setProxyConfig(proxyConfig);
					Logger.info(I18N.tr("Utils.setProxySettingsTo", proxyConfig.toString())); //$NON-NLS-1$
					return;
				}
			}
			Logger.error(I18N.tr("Utils.noDefaultProxy")); //$NON-NLS-1$
		} catch (URISyntaxException e) {
			Logger.error(I18N.tr("Utils.uriSyntaxException"), e); //$NON-NLS-1$
		}
	}

	/**
	 * Versucht die Proxy-Einstellungen manuell zu setzten. Evtl. Password wird
	 * im Hibicus Wallet gespeichert.
	 * 
	 * @param webClient
	 *            der WebClient
	 */
	private static void useManualProxy(final WebClient webClient) {

		final String host = Application.getConfig().getHttpsProxyHost();
		final int port = Application.getConfig().getHttpsProxyPort();
		if (host != null && host.length() > 0 && port > 0) {
			final ProxyConfig proxyConfig = new ProxyConfig();
			proxyConfig.setProxyHost(host);
			proxyConfig.setProxyPort(port);
			webClient.getOptions().setProxyConfig(proxyConfig);
			Logger.info(I18N.tr("Utils.setProxySettingsTo", proxyConfig.toString())); //$NON-NLS-1$

			final DefaultCredentialsProvider scp = new DefaultCredentialsProvider();
			// set proxy user name and password
			try {

				final Wallet wal = Settings.getWallet();

				Login login = (Login) wal.get("PROXY_PASS_MUENST"); //$NON-NLS-1$
				if (login == null) {
					login = Application.getCallback().login(Utils.J_AUTHENTICATOR);
					wal.set("PROXY_PASS_MAHOE", login); //$NON-NLS-1$
				}

				Logger.info(I18N.tr("Utils.user", login.getUsername())); //$NON-NLS-1$
				scp.addCredentials(login.getUsername(), String.valueOf(login.getPassword()), host, port, null);
				scp.addCredentials(login.getUsername(), String.valueOf(login.getPassword()));
				webClient.setCredentialsProvider(scp);

			} catch (Exception e) { // NOPMD
				Logger.error(I18N.tr("Utils.manualSettingFailed"), e); //$NON-NLS-1$
			}
		}

	}

	/**
	 * Remove EUR and tausend separator from given double string. Replaces
	 * German , to . to represent double in good format.
	 * 
	 * @param doubleString
	 *            der String der Konvertiert werden soll
	 * @return die konvertierte Zahl
	 */
	public static Double convertStringToDouble(final String doubleString) {
		Logger.debug("convertStringToDouble(" + doubleString + ")"); //$NON-NLS-1$ //$NON-NLS-2$
		String doubleStr = doubleString.replace("- ", "-"); //$NON-NLS-1$ //$NON-NLS-2$
		doubleStr = doubleStr.replace("EUR", StringUtils.EMPTY); //$NON-NLS-1$
		doubleStr = doubleStr.replace(".", StringUtils.EMPTY); //$NON-NLS-1$
		doubleStr = doubleStr.replace(",", "."); //$NON-NLS-1$ //$NON-NLS-2$
		doubleStr = doubleStr.replaceAll("\\s", StringUtils.EMPTY); //$NON-NLS-1$
		return Double.valueOf(doubleStr);
	}

	/**
	 * Formatiert einen Double-Wert in einen String mit genau 2
	 * Nachkommastellen.
	 * 
	 * @param doubleValue
	 *            der Double-Wert der Konvertiert werden soll.
	 * @return die konvertierte Zahl als String
	 */
	public static String convertDoubleToString(final Double doubleValue) {
		return Utils.DECIMAL_FORMAT.get().format(doubleValue);
	}

	/**
	 * Dezimal-Format mit genau 2 Nachkommastellen
	 */
	private static final ThreadLocal<DecimalFormat> DECIMAL_FORMAT = new ThreadLocal<DecimalFormat>() {
		/**
		 * @see ThreadLocal#initialValue()
		 */
		@Override
		protected DecimalFormat initialValue() {
			final DecimalFormatSymbols otherSymbols = new DecimalFormatSymbols(Locale.GERMAN);
			otherSymbols.setDecimalSeparator(',');
			otherSymbols.setGroupingSeparator('.');
			return new DecimalFormat("########0.00", otherSymbols); //$NON-NLS-1$
		}
	};

	/**
	 * Ld einen Umsatz-Beschreibungstext in einen Umsatz. Bercksichtigt dabei
	 * die max. Lngen fr die Zweck- und Zweck2-Felder und teilt die
	 * Beschreibung ggf. auf.
	 * 
	 * @param umsatz
	 *            das Hibicus Umsatz Objekt
	 * @param umsatzString
	 *            die Umsatzbeschreibung
	 * @throws RemoteException
	 */
	public static void loadUmsatz(final Umsatz umsatz, final String umsatzString) throws RemoteException {

		final StringBuffer strUmsatz = new StringBuffer();

		// split to rows
		final String[] rows = umsatzString.trim().split("\\r?\\n|\\r"); //$NON-NLS-1$
		Logger.debug("clearUmsatz[rows]: " + rows.length); //$NON-NLS-1$
		for (String row : rows) {

			if (StringUtils.isNotEmpty(row)) {
				Logger.debug("clearUmsatz[s]: " + row); //$NON-NLS-1$
				do {
					if (row.length() > Utils.MAX_ROW_LENGTH) {
						strUmsatz.append(row.substring(0, 34)).append('\n');
						row = row.substring(34);
					} else {
						strUmsatz.append(row).append('\n');
						row = StringUtils.EMPTY;
					}
				} while (row.length() > 0);
			}
		}

		final String[] umsatzZeilen = strUmsatz.toString().split("\n"); //$NON-NLS-1$

		switch (umsatzZeilen.length) {
		case 1:
			umsatz.setZweck(umsatzZeilen[0]);
			break;
		case 2:
			umsatz.setZweck(umsatzZeilen[0]);
			umsatz.setZweck2(umsatzZeilen[1]);
			break;
		default:
			umsatz.setZweck(umsatzZeilen[0]);
			umsatz.setZweck2(umsatzZeilen[1]);
			final String[] umsatz3 = Arrays.copyOfRange(umsatzZeilen, 2, umsatzZeilen.length);
			umsatz.setWeitereVerwendungszwecke(umsatz3);
			break;
		}

	}

	/**
	 * Liefert das Datum, ab wann die Umstze geladen werden sollen. Das ist
	 * entweder das Datum der letzten Buchung oder das angegebene Startdatum.
	 * Ist das Startdatum NULL, wird es auf den 01.01.2000 gesetzt.
	 * Zurckgegeben wird dann das jnger Datum.
	 * 
	 * @param konto
	 *            das Konto
	 * @param startDate
	 *            das StartDatum als Calendar-Objekt
	 * @return Das Abrufdatum, das benutzt werden soll.
	 */
	public static Date getLastUmsatzDate(final Konto konto, final Calendar startDate) {

		Date callDate = null;

		if (konto != null) {
			Date lastCall = new Date(0);
			try {
				final DBIterator<?> umsaetze = konto.getUmsaetze();
				while (umsaetze.hasNext()) {
					final Umsatz umsatz = (Umsatz) umsaetze.next();
					if (umsatz.getFlags() == Umsatz.FLAG_NOTBOOKED) {
						continue;
					}
					lastCall = umsatz.getDatum();
					break; // NOPMD
				}
			} catch (RemoteException e) {
				// Hier brauchen wir erst mal nix zu tun
				Logger.debug("Hmm, Umstze wurden nicht geladen. Ob das spter gut geht?"); //$NON-NLS-1$
			}

			// Setzt Startdatum auf angegebenes Datum. Falls Startdatum NULL
			// ist, setzten wir 01.01.2000
			Calendar startCal;
			if (startDate == null) {
				startCal = Calendar.getInstance();
				startCal.set(2000, 0, 0);
			} else {
				startCal = startDate;
			}

			// Ermittle das jngere Datum
			final Date maxOldDate = new Date(startCal.getTimeInMillis());
			String callDateAsString;
			final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", //$NON-NLS-1$
					Locale.GERMAN);
			if (lastCall.compareTo(maxOldDate) < 0) {
				callDate = maxOldDate;
				callDateAsString = sdf.format(maxOldDate);
				Logger.info(I18N.tr("Utils.firstCallLoadfrom", callDateAsString)); //$NON-NLS-1$
			} else {
				callDate = lastCall;
				callDateAsString = sdf.format(lastCall);
				Logger.info(I18N.tr("Utils.loadFrom", callDateAsString)); //$NON-NLS-1$
			}
		}
		// return date
		return callDate;

	}

	/**
	 * Rundet auf die angegebene Anzahl Nachkommastellen
	 * 
	 * @param number
	 *            die Zahl
	 * @param digits
	 *            die Anzahl an Nachkommastellen
	 * @return die gerundete Zahl
	 */
	public static double roundNumber(final double number, final int digits) {
		final double multiple = Math.pow(10, digits);
		return Math.round(number * multiple) / multiple;
	}

	/**
	 * 
	 * Scans all classes accessible from the context class loader which belong
	 * to the given package and subpackages.
	 * 
	 * @param packageName
	 *            The base package
	 * 
	 * @return The classes
	 * 
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	public static Class<?>[] getClasses(final String packageName) throws ClassNotFoundException, IOException {

		final ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
		assert classLoader != null;
		final String path = packageName.replace('.', '/');
		final Enumeration<URL> resources = classLoader.getResources(path);
		final List<File> dirs = new ArrayList<File>();

		while (resources.hasMoreElements()) {

			final URL resource = resources.nextElement();
			dirs.add(new File(resource.getFile())); // NOPMD
		}

		final ArrayList<Class<?>> classes = new ArrayList<Class<?>>();
		for (final File directory : dirs) {
			classes.addAll(Utils.findClasses(directory, packageName));
		}
		return classes.toArray(new Class[classes.size()]);
	}

	/**
	 * 
	 * Recursive method used to find all classes in a given directory and
	 * subdirs.
	 * 
	 * @param directory
	 *            The base directory
	 * @param packageName
	 *            The package name for classes found inside the base directory
	 * 
	 * @return The classes
	 * 
	 * @throws ClassNotFoundException
	 */

	private static List<Class<?>> findClasses(final File directory, final String packageName)
			throws ClassNotFoundException {

		final List<Class<?>> classes = new ArrayList<Class<?>>();
		if (directory.exists()) {
			final File[] files = directory.listFiles();
			for (final File file : files) {
				if (file.isDirectory()) {
					assert !file.getName().contains("."); //$NON-NLS-1$
					classes.addAll(Utils.findClasses(file, packageName + "." + file.getName())); //$NON-NLS-1$
				} else if (file.getName().endsWith(".class")) { //$NON-NLS-1$
					classes.add(Class.forName(packageName + '.'
							+ file.getName().substring(0, file.getName().length() - 6)));
				}
			}
		}
		return classes;
	}

}
