//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.web;

import java.util.List;

import de.frankmuenster.hibiscus.mahoe.messages.AbstractBankMessages;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.util.ApplicationException;

/**
 * Interface fr die Kommunikation mit der Bank-Webseite
 * 
 * @author Frank Mnster
 * 
 * @param <I>
 *            die bankspezifischen WebAccountInformation
 * @param <M>
 *            die bankspezifischen BankMessages
 * 
 */
public interface InterfaceWebSite<I extends InterfaceWebAccountInformation, M extends AbstractBankMessages> {

	/**
	 * berprft, ob das Webseite-Object immer noch an der Webseite angemeldet
	 * ist.
	 * 
	 * @return true wenn die Session noch aktiv ist, false sonst
	 * @throws WebSiteException
	 * 
	 */
	boolean isStillLoggedIn() throws WebSiteException;

	/**
	 * An der Bank-Seite anmelden
	 * 
	 * @param password
	 *            das Passwort
	 * 
	 * @param loginParams
	 *            Login-Parameter als Array, das verschiedene Banken
	 *            verschiedene Anforderungen haben knnen
	 * @throws WebSiteException
	 */
	void login(final char[] password, String... loginParams) throws WebSiteException;

	/**
	 * Abmelden von der Bank-Seite
	 * 
	 * @throws WebSiteException
	 */
	void logoff() throws WebSiteException;

	/**
	 * Liefert kontospezifische Informationen, die auf alle Flle das Saldo und
	 * das letzte Umsatzdatum enthalten.
	 * 
	 * @return die kontospezifische Informationen
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	I getAccountInformation() throws WebSiteException, ApplicationException;

	/**
	 * Liefert eine Liste mit den vorgemerkten Umstzen.
	 * 
	 * @return Liste mit den gefunden Umstzen, eine leere Liste sonst.
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException;

	/**
	 * Liefert eine Liste mit den gebuchten Umstzen.
	 * 
	 * @return Liste mit den gefunden Umstzen, eine leere Liste sonst.
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException;

	/**
	 * Ldt neue Bank-Nachrichten
	 * 
	 * @param accountInfo
	 * @return die Anzahl an gespeicherten Nachrichten
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	int getMessages(final I accountInfo) throws WebSiteException, ApplicationException;

	/**
	 * Speichert die Kontoauszge im angegeben Ordner.
	 * 
	 * @param accountInfo
	 *            die Kontoinformationen
	 * @return die Anzahl an gespeicherten Kontoauszge
	 * 
	 * @throws PdfSaverException
	 */
	int savePDF(final I accountInfo) throws PdfSaverException;

	/**
	 * Erstellt eine Sepa-berweisung bei der Bank.
	 * 
	 * @param aUeberweisung
	 *            die Sepa-berweisung.
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	void createSepaTransfer(final AuslandsUeberweisung aUeberweisung) throws WebSiteException, ApplicationException;

}
